#!/bin/bash

# Below are the BMC GPIO settings to access BIOS SPI flash
# GPIOs: [HPM_BMC_GPIOO1:HPM_BMC_GPIOR3]
#       [00] = Run Mode
#       [01] = Local BIOS SPI
#       [10] = HPM FPGA SPI
#       [11] = HPM LOM SPI
#

set +e

#############
# Functions #
#############
# Websocket message

# Progress, State and Error code
update_progress_and_state() {
    local VID=$1
    local PG=$2
    local STATE=$3
    local ERR=$4

    # check whether it exists or not
    busctl get-property xyz.openbmc_project.Software.BMC.Updater \
    /xyz/openbmc_project/software/$VID \
    xyz.openbmc_project.Software.ActivationProgress Progress > /dev/null

    if [ $? -eq 0 ]; then
        echo "[$STATE][$PG]"

        #dbus-send --system --type=signal --dest=xyz.openbmc_project.Software.BMC.Updater \
        #/xyz/openbmc_project/software/${VID} xyz.openbmc_project.Software.ActivationProgress.UpdateProgressSignal \
        #uint32:${PG} string:"${STATE}" int32:${ERR}

        busctl set-property xyz.openbmc_project.Software.BMC.Updater \
        /xyz/openbmc_project/software/bios_active \
        xyz.openbmc_project.Software.Version UpdateProgress s ${PG}

        busctl set-property xyz.openbmc_project.Software.BMC.Updater \
        /xyz/openbmc_project/software/bios_active \
        xyz.openbmc_project.Software.Version UpdateStatus s ${STATE}

        busctl set-property xyz.openbmc_project.Software.BMC.Updater \
        /xyz/openbmc_project/software/${VID} \
        xyz.openbmc_project.Software.ActivationProgress Progress y ${PG}

        busctl set-property xyz.openbmc_project.Software.BMC.Updater \
        /xyz/openbmc_project/software/${VID} \
        xyz.openbmc_project.Software.Activation StateDescription s ${STATE}
    fi
}

PG_FILE="/tmp/flash-progress"
function bios_upd_status ()
{
    VID=$1
    FILE=$2

    while [ 1 ]; do
        LOG=`cat ${FILE}|sed 's/\r/\n/g'|tail -n 1`
        STATUS=`echo $LOG | cut -d " " -f 1`
        PROGRESS=`echo $LOG | cut -d " " -f 4 | sed 's/(//g' | sed 's/)//g' | sed 's/%//g'`
        if [ ${STATUS} == "Verifying" ]; then
            # prevent phosphor-sw-mgr segmentation fault, becasue related dbus interface has been removed
            if [ ${PROGRESS} == "98" -o ${PROGRESS} == "99" -o ${PROGRESS} == "100" ]; then
                break
            fi
        fi
        update_progress_and_state $VID $PROGRESS $STATUS 1
    done
}

POWER_CMD_OFF="busctl set-property xyz.openbmc_project.State.Chassis /xyz/openbmc_project/state/chassis0 xyz.openbmc_project.State.Chassis RequestedPowerTransition s xyz.openbmc_project.State.Chassis.Transition.Off"
POWER_CMD_ON="busctl set-property xyz.openbmc_project.State.Chassis /xyz/openbmc_project/state/chassis0 xyz.openbmc_project.State.Chassis RequestedPowerTransition s xyz.openbmc_project.State.Chassis.Transition.On"
UPDATE_PROGRESS_FINISH="busctl set-property xyz.openbmc_project.Software.BMC.Updater /xyz/openbmc_project/software/bios_active xyz.openbmc_project.Software.Version UpdateProgress s 100"
UPDATE_STATUS_FAIL="busctl set-property xyz.openbmc_project.Software.BMC.Updater /xyz/openbmc_project/software/bios_active xyz.openbmc_project.Software.Version UpdateStatus s Failed"
IMAGE_DIR=$1

GPIOCHIP=816
GPIOR3=955

UPDATE_STATUS=0

# Lanai
GPIOV0=$((${GPIOCHIP} + 168 + 0))
GPIOV1=$((${GPIOCHIP} + 168 + 1))
LANAI_SPI_DEV="1e630000.spi"

SPI_DEV="1e630000.spi"
SPI_PATH="/sys/bus/platform/drivers/aspeed-smc"

IPMI_CMD="busctl call xyz.openbmc_project.Ipmi.Host /xyz/openbmc_project/Ipmi xyz.openbmc_project.Ipmi.Server execute yyyaya{sv}"
PRIVILEGE_PARM="2 "privilege" i 4 "userId" i 0"

power_status() {
    st=$(busctl get-property xyz.openbmc_project.State.Chassis /xyz/openbmc_project/state/chassis0 xyz.openbmc_project.State.Chassis CurrentPowerState | cut -d"." -f6)
    if [ "$st" == "On\"" ]; then
        echo "on"
    else
        echo "off"
    fi
}

set_gpio_to_bmc()
{
    echo "switch bios GPIO to bmc"
    if [ ! -d /sys/class/gpio/gpio$GPIOR3 ]; then
        cd /sys/class/gpio
        echo $GPIOR3 > export
        cd gpio$GPIOR3
    else
        cd /sys/class/gpio/gpio$GPIOR3
    fi
    direc=`cat direction`
    if [ $direc == "in" ]; then
        echo "out" > direction
    fi
    data=`cat value`
    if [ "$data" == "1" ]; then
        echo 0 > value
    fi
    return 0
}
set_lanai_gpio_to_bmc()
{
    echo "switch bios GPIO to bmc"
    if [ ! -d /sys/class/gpio/gpio$GPIOV0 ]; then
        cd /sys/class/gpio
        echo $GPIOV0 > export
        cd gpio$GPIOV0
    else
        cd /sys/class/gpio/gpio$GPIOV0
    fi
    direc=`cat direction`
    if [ $direc == "in" ]; then
        echo "out" > direction
    fi
    data=`cat value`
    if [ "$data" == "0" ]; then
        echo 1 > value
    fi
    if [ ! -d /sys/class/gpio/gpio$GPIOV1 ]; then
        cd /sys/class/gpio
        echo $GPIOV1 > export
        cd gpio$GPIOV1
    else
        cd /sys/class/gpio/gpio$GPIOV1
    fi
    direc=`cat direction`
    if [ $direc == "in" ]; then
        echo "out" > direction
    fi
    data=`cat value`
    if [ "$data" == "0" ]; then
        echo 1 > value
    fi
    return 0
}
set_gpio_to_host()
{
    echo "switch bios GPIO to host"
    if [ ! -d /sys/class/gpio/gpio$GPIOR3 ]; then
        cd /sys/class/gpio
        echo $GPIOR3 > export
        cd gpio$GPIOR3
    else
        cd /sys/class/gpio/gpio$GPIOR3
    fi
    direc=`cat direction`
    if [ $direc == "in" ]; then
        echo "out" > direction
    fi
    data=`cat value`
    if [ "$data" == "0" ]; then
        echo 1 > value
    fi
    echo "in" > direction
    echo $GPIOR3 > /sys/class/gpio/unexport


    return 0
}
set_lanai_gpio_to_host()
{
    echo "switch bios GPIO to host"
    if [ ! -d /sys/class/gpio/gpio$GPIOV0 ]; then
        cd /sys/class/gpio
        echo $GPIOV0 > export
        cd gpio$GPIOV0
    else
        cd /sys/class/gpio/gpio$GPIOV0
    fi
    direc=`cat direction`
    if [ $direc == "in" ]; then
        echo "out" > direction
    fi
    data=`cat value`
    if [ "$data" == "1" ]; then
        echo 0 > value
    fi
    echo "in" > direction
    echo $GPIOV0 > /sys/class/gpio/unexport
    if [ ! -d /sys/class/gpio/gpio$GPIOV1 ]; then
        cd /sys/class/gpio
        echo $GPIOV1 > export
        cd gpio$GPIOV1
    else
        cd /sys/class/gpio/gpio$GPIOV1
    fi
    direc=`cat direction`
    if [ $direc == "in" ]; then
        echo "out" > direction
    fi
    data=`cat value`
    if [ "$data" == "1" ]; then
        echo 0 > value
    fi
    echo "in" > direction
    echo $GPIOV1 > /sys/class/gpio/unexport
    return 0
}

flash_image_to_mtd() {
    # Get VID of phosphor-sw-mgr
    IMAGE_LOC=${IMAGE_DIR%/*}
    VID=${IMAGE_LOC##*/}
    echo $IMAGE_DIR
    pushd $IMAGE_DIR
    IMAGE_FILE=$(find -type f -name '*.FD' -o -name 'image-bios')
    if [ -e "$IMAGE_FILE" ];
    then
        echo "Bios image is $IMAGE_FILE"
        for d in mtd{6..20}; do
            if [ -e "/dev/$d" ]; then
                mtd=`cat /sys/class/mtd/$d/name`
                if [ $mtd == "pnor" ]; then
                    echo "Flashing bios image to $d..."
                    rm -rf $PG_FILE
                    rm /tmp/progress.flag
                    bios_upd_status $VID $PG_FILE &
                    flashcp -v $IMAGE_FILE /dev/$d > $PG_FILE
                    #flash_eraseall /dev/$d
                    #dd if=$IMAGE_FILE of=/dev/$d bs=4096
                    if [ $? -eq 0 ]; then
                        echo "bios updated successfully..."
                        UPDATE_STATUS=0
                    else
                        echo "bios update failed..."
                        UPDATE_STATUS=1
                    fi
                    break
                fi
                echo "$d is not a pnor device"
                UPDATE_STATUS=1
            fi
            echo "$d not available"
            UPDATE_STATUS=1
        done
    else
        echo "Bios image $IMAGE_FILE doesn't exist"
        UPDATE_STATUS=1
    fi
    popd
}

exit_bios_update_mode() {
    # For HPM update to use
    ${IPMI_CMD} 0x3E 0x00 0xC0 2 0x08 0x11 ${PRIVILEGE_PARM}
}


switch_to_BIOS_flash2(){
    echo "switch to flash 2"
    i2cget -y 4 0x17 0x04
    sleep 1
    i2cset -y 4 0x17 0x04 0x81
    sleep 1
    i2cget -y 4 0x17 0x04
    sleep 1
}

switch_back_to_BIOS_flash1(){
    echo "switch back to flash 1"
    i2cget -y 4 0x17 0x04
    sleep 1
    i2cset -y 4 0x17 0x04 0x80
    sleep 1
    i2cget -y 4 0x17 0x04
    sleep 1
    i2cset -y 4 0x17 0x04 0x00
    sleep 1
}

echo "Bios upgrade started at $(date)"
power_state=$(power_status)
echo "Current Host state is $power_state"

# Kevin.Yeh 2024-08-01: Allow BIOS update during power on.
# if [ "$power_state" != "off" ];
# then
# #Power off host server.
# echo "Power off host server"
# $POWER_CMD_OFF
# sleep 10
# fi

# if [ $(power_status) != "off" ];
# then
#     echo "Host server didn't power off"
#     echo "Bios upgrade failed"
#     exit_bios_update_mode
#     exit -1
# fi
# echo "Host server powered off"

type=`cat /tmp/UpdateImageType`

touch /tmp/progress.flag

#Flip GPIO to access SPI flash used by host.
echo "Set GPIO $GPIO to access SPI flash from BMC used by host"
set_gpio_to_bmc

if [ "$type" == "1" ]; then
    echo "Choose flash BIOS 1"
    echo "Now update BIOS flash1"
elif [ "$type" == "2" ]; then
    echo "Choose flash BIOS 2"
    switch_to_BIOS_flash2
    echo "Now update BIOS flash2"

else
    echo "Choose flash BIOS 1 & 2"
    echo "Now update BIOS flash1"
fi
#Bind spi driver to access flash
echo "bind aspeed-smc spi driver"
#Bind spi driver to access flash
echo -n $SPI_DEV > $SPI_PATH/bind

if [ $? -eq 0 ];
then
    echo "SPI Driver Bind Successful"
else
    echo "SPI Driver Bind Failed.Run micron_v2.ini or micron_v3.ini using DediProg."
    set_gpio_to_host
    sleep 5
    exit_bios_update_mode
    exit -1
fi
sleep 1

#Flashcp image to device.
flash_image_to_mtd

#Unbind spi driver
sleep 10
echo "Unbind aspeed-smc spi driver"
echo -n $SPI_DEV > $SPI_PATH/unbind
sleep 1
if [ "$type" == "2" ]; then
    switch_back_to_BIOS_flash1
    echo "finish update BIOS flash 2"
else
    echo "finish update BIOS flash 1"
fi

if [ "$type" == "3" ]; then
    echo "Now update BIOS flash2"
    switch_to_BIOS_flash2
    #Bind spi driver to access flash
    echo "bind aspeed-smc spi driver"
    #Bind spi driver to access flash
    echo -n $SPI_DEV > $SPI_PATH/bind
    if [ $? -eq 0 ];
    then
        echo "SPI Driver Bind Successful"
    else
        echo "SPI Driver Bind Failed.Run micron_v2.ini or micron_v3.ini using DediProg."
        set_gpio_to_host
        sleep 5
        exit_bios_update_mode
        exit -1
    fi
    sleep 1
    #Flashcp image to device.
    flash_image_to_mtd
    #Unbind spi driver
    sleep 10
    echo "Unbind aspeed-smc spi driver"
    echo -n $SPI_DEV > $SPI_PATH/unbind
    sleep 1
    echo "finish update BIOS flash 2"
    switch_back_to_BIOS_flash1
fi




#Flip GPIO back for host to access SPI flash
echo "Set GPIO $GPIO back for host to access SPI flash"
set_gpio_to_host
sleep 5
if [ "$UPDATE_STATUS" -eq "0" ]; then
    echo "Update pass"
    $UPDATE_PROGRESS_FINISH
    sleep 1
else
    echo "Update fail"
    $UPDATE_STATUS_FAIL
    sleep 1
fi

# Kevin.Yeh 2024-08-01: Allow BIOS update during power on.
# if [ "$power_state" == "on" ];
# then
# $POWER_CMD_ON
# sleep 1
# fi

# For HPM update to use
# ipmitool raw 0x3e 0xc0 0x8 0x31 -> set status of BIOSUPD_FlashThisBlock_FromFile_Set_Status to IDLE
sleep 1
${IPMI_CMD} 0x3E 0x00 0xC0 2 0x08 0x31 ${PRIVILEGE_PARM}
sleep 1
exit_bios_update_mode

exit 0

